"""CLI helpers."""


from beartype import beartype

from .. import __version__
from ..tangs import registered_tangs


@beartype
def task_help() -> None:
    """Print the help text."""
    print(f'shoal ({__version__}): python-first task runner 🐠')

    tangs = registered_tangs().values()
    if tangs:
        print('')

    for tang in tangs:
        # TODO: Pretty-print help with rich
        print(f'{tang.target}\n\t{tang.description}')

    print('')



"""Tang (task model).

```sh
  _
><_>
```

"""

from beartype.typing import Callable, List
from beartype import beartype
from pydantic import BaseModel


class Tang(BaseModel):
    """A singular task (named after the Tang fish).

    The inner naming and logic derives from Makefiles: https://www.gnu.org/software/make/manual/html_node/Introduction.html

    ```sh
    .PHONY target
    target: prerequisites
    <TAB> recipe
    ```

    """

    target: str
    """Name of the task."""

    recipe: List[Callable[[List[str]], None]]
    """Steps in the task."""

    description: str = ''
    """Optional help text."""

    # > PLANNED: Add support for these additional arguments

    # prerequisities: List[str] = Field(default_factory=list)
    # """Optional task and/or file prerequisities."""

    # phony: bool = False
    # """Set to True if the `target` should *not* be included in the file preqreuisites.
    #
    # By default, assumes that the target name is associated with the matching file.
    #
    # https://stackoverflow.com/a/2145605/3219667
    #
    # """

    @beartype
    def run(self, args: List[str]) -> None:
        for step in self.recipe:
            step(args)



"""Tangs (the collection of tasks)."""

from beartype import beartype
from beartype.typing import Dict
from pydantic import BaseModel, Field

from .tang import Tang


class Tangs(BaseModel):
    """Dynamic collection of targets."""

    targets: Dict[str, Tang] = Field(default_factory=dict)
    """All registered targets."""

    @beartype
    def add_tang(self, tang: Tang) -> None:
        """Dynamically add a new Tang to the target list."""
        if tang.target in self.targets:
            print(f'Replaceing {self.targets[tang.target]} with {tang}')
        self.targets[tang.target] = tang


_TANGS = Tangs()
"""Internal state for global tangs."""


@beartype
def register(tang: Tang) -> None:
    """Dynamically add a new Tang to the target list."""
    _TANGS.add_tang(tang)


@beartype
def registered_tangs() -> Dict[str, Tang]:
    """Retrieve the registered tangs."""
    return _TANGS.targets



"""shoal."""

from warnings import filterwarnings

from beartype.roar import BeartypeDecorHintPep585DeprecationWarning
from loguru import logger

__version__ = '0.1.0'
__pkg_name__ = 'shoal'

logger.disable(__pkg_name__)

# ====== Above is the recommended code from calcipy_template and may be updated on new releases ======

# FYI: https://github.com/beartype/beartype#are-we-on-the-worst-timeline
filterwarnings('ignore', category=BeartypeDecorHintPep585DeprecationWarning)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['shoal', 'shoal._private']

package_data = \
{'': ['*']}

install_requires = \
['beartype>=0.12.0', 'calcipy>=0.21.3', 'pydantic>=1.10.4']

setup_kwargs = {
    'name': 'shoal',
    'version': '0.1.0',
    'description': 'Lazy, Python-first, alternative to make, just, doit, and invoke',
    'long_description': '# shoal\n\nPython function and shell command task runner. Alternative to `make`, [`just`](https://github.com/casey/just), [`doit`](https://github.com/pydoit/doit), and [`invoke`](https://pypi.org/project/invoke). Used in [`calcipy`](https://pypi.org/project/calcipy)\n\n## Installation\n\n`poetry add shoal`\n\n## Usage\n\nThere are two ways to use `shoal`, either as a way to organize a set of local tasks or to distribute with a package.\n\nFor more example code, see the [scripts] directory or the [tests].\n\n### Local Runner\n\nCreate a `shoal.py` file with:\n\n```py\nfrom shoal.link import shoalling\n\nshoalling()\n```\n\nThen run the file with:\n\n```sh\npoetry run python shoal.py --task-help\n```\n\nAdd additional tasks or import tasks from a third party package (such as [`calcipy`](https://pypi.org/project/calcipy))\n\n### Package\n\nCreate a `<package>/shoal.py` file with:\n\n```py\nfrom shoal.link import shoalling  # noqa: F401\n```\n\nAdd the file to your `pyproject.toml` scripts:\n\n```toml\n[tool.poetry.scripts]\nreplace_me = "shoal:shoalling"\n```\n\nThen test with:\n\n```sh\npoetry run replace_me --task-help\n```\n\n## Project Status\n\nSee the `Open Issues` and/or the [CODE_TAG_SUMMARY]. For release history, see the [CHANGELOG].\n\n## Contributing\n\nWe welcome pull requests! For your pull request to be accepted smoothly, we suggest that you first open a GitHub issue to discuss your idea. For resources on getting started with the code base, see the below documentation:\n\n- [DEVELOPER_GUIDE]\n- [STYLE_GUIDE]\n\n## Code of Conduct\n\nWe follow the [Contributor Covenant Code of Conduct][contributor-covenant].\n\n### Open Source Status\n\nWe try to reasonably meet most aspects of the "OpenSSF scorecard" from [Open Source Insights](https://deps.dev/pypi/shoal)\n\n## Responsible Disclosure\n\nIf you have any security issue to report, please contact the project maintainers privately. You can reach us at [dev.act.kyle@gmail.com](mailto:dev.act.kyle@gmail.com).\n\n## License\n\n[LICENSE]\n\n[changelog]: ./docs/CHANGELOG.md\n[code_tag_summary]: ./docs/CODE_TAG_SUMMARY.md\n[contributor-covenant]: https://www.contributor-covenant.org\n[developer_guide]: ./docs/DEVELOPER_GUIDE.md\n[license]: https://github.com/kyleking/shoal/LICENSE\n[scripts]: https://github.com/kyleking/shoal/scripts\n[style_guide]: ./docs/STYLE_GUIDE.md\n[tests]: https://github.com/kyleking/shoal/tests\n',
    'author': 'Kyle King',
    'author_email': 'dev.act.kyle@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/kyleking/shoal',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8.12,<4.0.0',
}


setup(**setup_kwargs)



"""Main 'shoalling' interface."""

import argparse
import sys

from beartype import beartype
from beartype.typing import List

from ._private.cli import task_help
from .tangs import registered_tangs


@beartype
def _swim(shoal_args: List[str]) -> None:
    """Run each Tang."""
    tangs = registered_tangs()

    argv = set(shoal_args)
    targets = argv.intersection(set(tangs))
    if not targets:
        raise ValueError(f'No tangs were specified in input. Excepted one of: {[*tangs]}')

    path_args = [*argv - targets]
    for target in targets:
        tangs[target].run(path_args)


@beartype
def _run() -> None:  # pragma: no cover
    """CLI Entrypoint."""
    # PLANNED: Add a flag (--debug & store_true) to print debugging information

    parser = argparse.ArgumentParser(description='shoal runner')
    parser.add_argument('-t', '--task-help', action='store_true', help='Print help for tasks')
    parser.add_argument('shoal_args', help='Arguments passed to shoal. See "-t" for more', nargs='*')
    options = parser.parse_args(sys.argv[1:])

    if options.task_help:
        task_help()
    else:
        _swim(options.shoal_args)


@beartype
def shoalling() -> None:  # pragma: no cover
    """Add to a 'shoal.py' file to run `shoal`."""
    _run()
